#-------------------------------------------------------------------------------
#
# Thomas Thomassen
# thomas[at]thomthom[dot]net
#
#-------------------------------------------------------------------------------

module TT::Plugins::QuadFaceTools

  # TT::Plugins::QuadFaceTools.convert_connected_mesh_to_quads2
  #
  # @since 0.7.0
  def self.convert_connected_mesh_to_quads2
    # <debug>
    t = Time.now
    # </debug>
    model = Sketchup.active_model
    selection = model.selection
    # Verify selection.
    if selection.size == 1 && selection[0].is_a?( Sketchup::Edge )
      edge = selection[0]
      if edge.faces.size == 2 && edge.faces.all? { |f| f.edges.size == 3 }
        faces = edge.faces
      else
        return UI.messagebox( 'Selection must contain one or two faces representing a QuadFace or an edge separating two triangles.' )
      end
    else
      faces = selection.select { |e| e.is_a?( Sketchup::Face ) }
      unless  ( faces.size == 1 && faces.all? { |f| f.edges.size == 4 } ) ||
              ( faces.size == 2 && faces.all? { |f| f.edges.size == 3 } )
        return UI.messagebox( 'Selection must contain one or two faces representing a QuadFace or an edge separating two triangles.' )
      end
    end
    # Process mesh.
    TT::Model.start_operation( 'Convert Mesh to QuadFaces' )
    converter = MeshToQuadConverter.new( *selection.to_a )
    model.commit_operation
    # <debug>
    TT.debug "self.convert_connected_mesh_to_quads: #{Time.now - t}"
    # </debug>
  end
  
  
  # @since 0.7.0
  class MeshToQuadConverter
    
    # @overload new( native_quad )
    #   @param [Sketchup::Face] native_quad
    # @overload new( triangle1, triangle2 )
    #   @param [Sketchup::Face] triangle1
    #   @param [Sketchup::Face] triangle2
    #
    # @since 0.7.0
    def initialize( *args )
      @entities = EntitiesProvider.new
      quad = @entities.convert_to_quad( *args )
    end
    
    # @param [QuadFace] origin
    #
    # @since 0.7.0
    def find_connected_quads( origin )
      done = origin.faces # Processed faces.
      triangles = [] # Unknown.
      quads = [] # Known.
      
      # Find confirmed quadfaces connected each vertex. (Native or QuadFace)
      for vertex in origin.vertices
        for face in @entities.get( vertex.faces )
          next if done.include?( face )
          if face.is_a?( QuadFace )
            done.concat( face.faces )
            quads << face
          elsif face.vertices.size == 3
            triangles << face
          end # if QuadFace.is?( face )
        end # for face
      end # for vertex
      
      # Neighbours
      neighbours = []
      for edge in origin.edges
        for face in edge.faces
          next if origin.faces.include?( face )
          next unless face.vertices.size == 3
          neighbours << face
        end
      end
      
      # <debug>
      quadface_origin.material = 'pink'
      triangles.each { |f| f.material = [100,100,100] }
      quads.each { |f| f.material = [100,255,100] }
      # <debug/>
    end
    
  end # class MeshToQuadConverter

end # module